# -*- coding: utf-8 -*-

"""
A package of classes I find myself constantly writing
for separate projects. This is public mostly for my
own convenience and use, although others are free to
use and contribute, of course!

Source:
https://github.com/cmyui/cmyui_pkg

:copyright: (c) 2020 cmyui
:license: MIT, see LICENSE for details.
"""

__title__ = 'cmyui'
__author__ = 'cmyui'
__license__ = 'MIT'
__copyright__ = 'Copyright 2020 cmyui'
__version__ = '1.0.5'

from .connection import *
from .mysqlconn import *
from .packetstream import *
from .server import *
from .types import *
from .version import *
